/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		8-Bit-Analog-Digital-Converter mit ATMEL ATmega8 to STK500 board, output on (PortB) --> LEDs 

Description: 	The ADC converts an analog input voltage to a 10-bit digital value through successive 
				approximation. The minimum value represents GND and the maximum value represents the 
				voltage on the AREF

Author: 		Peniely Lydos

AppNote:        AVR500 - ADC implementation

Datum: 			26/05/2009

Version: 		0.0

File Name: 		ADC_USART_Init_Read_adc.c

++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

//----------------------------------------------------------------------------------------------------------------
//ADC Multiplexer Selecter Register (ADMUX)
//----------------------------------------------------------------------------------------------------------------
//REFS1		REFS0		ADLAR		-		MUX3	MUX2	MUX1	MUX0
//voltage reference for the ADC
//REFS1	REFS0	Wert
//  0	  0		AREF, Internal Vref turned off
//  0	  1		AVCC with external capacitor at AREF pin
//  1	  0		Reserved
//  1	  1		Internal 2.56V Voltage Reference with external capacitor at AREF pin
//						  1=> to left adjust the result
//						  0=> the result is right adjusted
//											{-Analog Channel Selection bits-}
//											0		0		0		0	ADC0	
//											0		0		0		1	ADC1
//											0		0		1		0	ADC2
//											0		0		1		1	ADC3
//											0		1		0		0	ADC4
//											0		1		0		1	ADC5
//											0		1		1		0	ADC6
//											0		1		1		1	ADC7
//----------------------------------------------------------------------------------------------------------------
//Externe Refferenzspannung;Ergebniss Recht und Kanal ADC0
//----------------------------------------------------------------------------------------------------------------
//ADC Controll und STATUS REGISTER A (ADCSRA)
//----------------------------------------------------------------------------------------------------------------
//  ADEN	ADSC	ADFR	ADIF	  ADIE	   ADPS2	ADPS1	   ADPS0
//1=>Writing this bit to one enables the ADC
//			1=>In Single Conversion mode, start each conversion. In Free Running mode start the first conversion.
//					1=>the ADC operates in Free Running mode
//							"1/0" 	=>STATUS will be set from Conversion whenn new Result in memory
//									"1/0"  => Status whenn I-Flag (SFR) und ADIE setting star the ADC routine
//											  {" ADC Prescaler Select Bits"} Division Factor
//												0		  0	  		0			2
//  											0		  0	  		1			2
//  											0		  1	  		0			4
//  										  	0		  1	  		1			8
//  										  	1 		  0	  		0			16
//  										  	1		  0	  		1			32		
//  										  	1		  1	  		0			64
//  										  	1		  1	  		1			128
//The ADC requires an input clock frequency between 50 kHz and 200 kHz to get maximum resolution.
//TF_min=CLK/200000
//TF_max=CLK/50000 


#ifndef F_CPU
/* In the new version of the WinAVR/Mfile Makefile guideline One can defined F_CPU in the Makefile, a repeated 
   definition here would lead to a compiler warning . therefore "prevention" through    #ifndef/#endif

   This "Prevention" can lead to Debugger, if AVRStudio use a another, not the hardware fitting Clock rate: 
   Then the    following definition doesn't use, but instead the default value (1 MHz?) of AVRStudio - hence 
   the Output of a warning if F_CPU yet does not define:*/
#warning "F_CPU was not defined yet, now make up with 4000000"
#define F_CPU 4000000L    							// Systemtakt in Hz - define as  long>> Without errors in the computation 
#endif

#include <avr/io.h>
#include <inttypes.h>
#include <util/delay.h>
#include <stdlib.h>
#include "ADC_USART_Read_adc.h"


void init(void) //Initialisierung
{
	//DDRB = 0xff; 	
	DDRC = 0x00;	//Define as Input
	PORTC = 0x1;    //Enables Pull-Up on PC0
	DDRD = 0xff;
}//End init()


void warte (void)
{
	for (unsigned int i = 40; i>0; i--);
}//End warte ()


void ADC_Init (uint8_t Mux_channel)
{
	ADCSRA = (1<<ADEN); 				//Enables the ADC
	ADCSRA |= (1<<ADPS2) | (1<<ADPS0); 	//Determine the division factor between the XTAL frequency and the input clock to the ADC. 
										//Activate ADC with Prescaler 32 --> 4Mhz/32 = 125kHz Tmin = 20 , Tmax = 80 
	
	ADMUX = Mux_channel;				//Select pin ADC0 using MUX
	_delay_us(40);
	//warte ();
	ADMUX |= (1<<REFS0) | (1<<REFS1);	//Int. REF Voltage Reference Selection
	ADMUX |= (0<<ADLAR);				//The result is right adjusted
}

	
uint16_t ADC_Read(uint8_t Mux_channel) 
{
	uint8_t i;
	uint16_t result = 0;	 
	

	 //Initialisierung ADC
  
  /*//interrupt
  TCNT0 = 5;  							//Zhlregister setzen
  TIMSK |= (1<<TOIE0);					//damit interrupt auch auslst wenn berlauf erreicht wird
  TCCR0 |= (1<<CS01);
  TCCR0 |= (1<<CS00);					//CPU Takt wird verwendet --> 4MHz
  TIFR |= (1<<TOV0);					//startet interrupt bei jedem berlauf */
  
	
	ADC_Init (Mux_channel);

	ADCSRA |= (1<<ADSC);				//Start conversion
	while(ADCSRA & (1<<ADSC));			//wait until converstion completed
	result = ADCW;						//ADCW must be read once
    result = 0;            				//Otherwise the result of the next transformation is not taken over
                  				
	
	//Now reads 4 times the similar tension and selected channel channel 
	//And then calculate the average value divided by 4. Because the ADC measeres 0..1023,
	//And the Port however can only 0..255 show
	for(i=0; i<4; i++) 
	{		
		ADCSRA |= (1<<ADSC);			//A single conversion		
		while(ADCSRA & (1<<ADSC));		//Waiting on conversion closing		
		result += ADCW;
	}
	
	ADCSRA &= (0<<ADEN);  				// ADC reanables	
	result /= 4;
		
	return result;
}//End int ADC_Read() 


